<?php
/**
 * Copyright 2023 © Pinpoint Designs. All rights reserved.
 */
namespace Pinpoint\FileUpload\Model\Cms\Wysiwyg\Images;

use Exception;
use Magento\Cms\Model\Wysiwyg\Images\Storage as BaseStorage;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\ValidatorException;
use Magento\Framework\Image\AdapterFactory;
use Magento\Framework\View\Asset\Repository;
use Magento\MediaStorage\Model\File\UploaderFactory;
use Psr\Log\LoggerInterface;
use Magento\Framework\Filesystem\DriverInterface;
use Magento\Framework\Filesystem\Io\File;
use Magento\Framework\File\Mime;
use Magento\Framework\App\Filesystem\DirectoryList;

class Storage extends BaseStorage
{
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @param \Magento\Backend\Model\Session $session
     * @param \Magento\Backend\Model\UrlInterface $backendUrl
     * @param \Magento\Cms\Helper\Wysiwyg\Images $cmsWysiwygImages
     * @param \Magento\MediaStorage\Helper\File\Storage\Database $coreFileStorageDb
     * @param \Magento\Framework\Filesystem $filesystem
     * @param AdapterFactory $imageFactory
     * @param Repository $assetRepo
     * @param BaseStorage\CollectionFactory $storageCollectionFactory
     * @param \Magento\MediaStorage\Model\File\Storage\FileFactory $storageFileFactory
     * @param \Magento\MediaStorage\Model\File\Storage\DatabaseFactory $storageDatabaseFactory
     * @param \Magento\MediaStorage\Model\File\Storage\Directory\DatabaseFactory $directoryDatabaseFactory
     * @param UploaderFactory $uploaderFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param array $resizeParameters
     * @param array $extensions
     * @param array $dirs
     * @param array $data
     * @param DriverInterface|null $file
     * @param File|null $ioFile
     * @param LoggerInterface|null $logger
     * @param Mime|null $mime
     * @param ScopeConfigInterface|null $coreConfig
     * @throws FileSystemException
     */
    public function __construct(
        \Magento\Backend\Model\Session $session,
        \Magento\Backend\Model\UrlInterface $backendUrl,
        \Magento\Cms\Helper\Wysiwyg\Images $cmsWysiwygImages,
        \Magento\MediaStorage\Helper\File\Storage\Database $coreFileStorageDb,
        \Magento\Framework\Filesystem $filesystem,
        AdapterFactory $imageFactory,
        Repository $assetRepo,
        \Magento\Cms\Model\Wysiwyg\Images\Storage\CollectionFactory $storageCollectionFactory,
        \Magento\MediaStorage\Model\File\Storage\FileFactory $storageFileFactory,
        \Magento\MediaStorage\Model\File\Storage\DatabaseFactory $storageDatabaseFactory,
        \Magento\MediaStorage\Model\File\Storage\Directory\DatabaseFactory $directoryDatabaseFactory,
        UploaderFactory $uploaderFactory,
        ScopeConfigInterface $scopeConfig, // New dependency for config
        array $resizeParameters = [],
        array $extensions = [],
        array $dirs = [],
        array $data = [],
        DriverInterface $file = null,
        File $ioFile = null,
        LoggerInterface $logger = null,
        Mime $mime = null,
        ScopeConfigInterface $coreConfig = null
    ) {
        $this->scopeConfig = $scopeConfig;
        parent::__construct(
            $session,
            $backendUrl,
            $cmsWysiwygImages,
            $coreFileStorageDb,
            $filesystem,
            $imageFactory,
            $assetRepo,
            $storageCollectionFactory,
            $storageFileFactory,
            $storageDatabaseFactory,
            $directoryDatabaseFactory,
            $uploaderFactory,
            $resizeParameters,
            $extensions,
            $dirs,
            $data,
            $file,
            $ioFile,
            $logger,
            $mime,
            $coreConfig
        );
    }


    /**
     * Uploads a file to the specified target path.
     *
     * @param $targetPath
     * @param $type
     * @return array|bool
     * @throws FileSystemException
     * @throws LocalizedException
     * @throws ValidatorException
     */
    public function uploadFile($targetPath, $type = null)
    {
        $uploader = $this->_uploaderFactory->create(['fileId' => 'image']);
        $allowedExtensions = $this->getAllowedExtensions($type);

        if ($allowedExtensions) {
            $uploader->setAllowedExtensions($allowedExtensions);
        }

        $uploader->setAllowRenameFiles(true);
        $uploader->setFilesDispersion(false);
        $result = $uploader->save($targetPath);

        if (!$result) {
            throw new LocalizedException(__('We can\'t upload the file right now.'));
        }

        $fileExtension = $uploader->getFileExtension();
        if (!in_array($fileExtension, $allowedExtensions)) {
            $this->resizeFile($targetPath . '/' . $uploader->getUploadedFileName(), true);
        }

        $result['cookie'] = [
            'name' => $this->getSession()->getName(),
            'value' => $this->getSession()->getSessionId(),
            'lifetime' => $this->getSession()->getCookieLifetime(),
            'path' => $this->getSession()->getCookiePath(),
            'domain' => $this->getSession()->getCookieDomain(),
        ];

        return $result;
    }

    /**
     * Get allowed file extensions from configuration.
     *
     * @param string|null $type
     * @return array
     */
    public function getAllowedExtensions($type = null)
    {
        $enabledFileTypes = $this->scopeConfig->getValue('pinpoint_fileupload/allowed_filetypes/enabled_filetypes', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $allowedExtensions = explode(',', $enabledFileTypes);
        return $allowedExtensions;
    }
}
